package meerkat.mixer.proofs;

import com.google.protobuf.ByteString;
import meerkat.protobuf.Mixing;
import org.bouncycastle.math.ec.ECPoint;
import org.factcenter.qilin.primitives.concrete.ECGroup;

import java.math.BigInteger;

/**
 * Created by Tzlil on 1/25/2016.
 * zero knowledge proof parser
 */
public class ZeroKnowledgeOrProofParser {

    private final ECGroup group;

    /**
     * parse or proof message and return the result in zero knowledge or proof container
     * @param orProof
     * @return zero knowledge or proof container
     */
    public ZeroKnowledgeOrProofContainer parseOrProof(Mixing.ZeroKnowledgeProof.OrProof orProof){
        return new ZeroKnowledgeOrProofContainer(orProof);
    }

    /**
     * getter
     * @param group
     */
    public ZeroKnowledgeOrProofParser(ECGroup group) {
        this.group = group;
    }

    /**
     * convert ByteString to ECPoint
     * @param bs
     * @return
     */
    public ECPoint convert2ECPoint(ByteString bs){
        return group.decode(bs.toByteArray());
    }


    /**
     * inner class
     * container for parsed zero knowledge or proof
     * constructor is private, can be construct using ZeroKnowledgeOrProofParser.parseOrProof
     */
    public class ZeroKnowledgeOrProofContainer{
        public final ECPoint g1,g2,h1,h2;
        public final ECPoint g1Tag,g2Tag,h1Tag,h2Tag;
        public final ECPoint u,v,uTag,vTag;
        public final BigInteger c1,c2,z,zTag;
        public final  Mixing.ZeroKnowledgeProof.OrProof.ForRandomOracle forRandomOracle;

        /**
         * constructor
         * @param orProof
         */
        private ZeroKnowledgeOrProofContainer(Mixing.ZeroKnowledgeProof.OrProof orProof){
            this.forRandomOracle =
                    Mixing.ZeroKnowledgeProof.OrProof.ForRandomOracle.newBuilder()
                            .setG1(orProof.getG1())
                            .setH1(orProof.getH1())
                            .setG2(orProof.getG2())
                            .setH2(orProof.getH2())
                            .setG1Tag(orProof.getG1Tag())
                            .setH1Tag(orProof.getH1Tag())
                            .setG2Tag(orProof.getG2Tag())
                            .setH2Tag(orProof.getH2Tag())
                            .setU(orProof.getU())
                            .setV(orProof.getV())
                            .setUTag(orProof.getUTag())
                            .setVTag(orProof.getVTag())
                            .build();
            this.g1 = convert2ECPoint(orProof.getG1());
            this.g2 = convert2ECPoint(orProof.getG2());
            this.h1 = convert2ECPoint(orProof.getH1());
            this.h2 = convert2ECPoint(orProof.getH2());
            this.g1Tag = convert2ECPoint(orProof.getG1Tag());
            this.g2Tag = convert2ECPoint(orProof.getG2Tag());
            this.h1Tag = convert2ECPoint(orProof.getH1Tag());
            this.h2Tag = convert2ECPoint(orProof.getH2Tag());
            this.u = convert2ECPoint(orProof.getU());
            this.v = convert2ECPoint(orProof.getV());
            this.uTag = convert2ECPoint(orProof.getUTag());
            this.vTag = convert2ECPoint(orProof.getVTag());
            this.c1 = new BigInteger(orProof.getC1().toByteArray());
            this.c2 = new BigInteger(orProof.getC2().toByteArray());
            this.z = new BigInteger(orProof.getZ().toByteArray());
            this.zTag = new BigInteger(orProof.getZTag().toByteArray());
        }
    }
}
